package com.me.asteroids;

import com.badlogic.gdx.Gdx;
import com.badlogic.gdx.graphics.Camera;
import com.badlogic.gdx.graphics.GL20;
import com.badlogic.gdx.graphics.OrthographicCamera;
import com.badlogic.gdx.graphics.glutils.ShapeRenderer;
import com.badlogic.gdx.utils.viewport.FitViewport;
import com.badlogic.gdx.utils.viewport.Viewport;

public class Graphics {

    private int worldWidth, worldHeight;
    private int screenWidth, screenHeight;

    private Camera camera;
    private Viewport viewport;

    private ShapeRenderer shapeRenderer;

    public Graphics(int worldWidth, int worldHeight) {
        this.worldWidth = worldWidth;
        this.worldHeight = worldHeight;
        this.screenWidth = Gdx.graphics.getWidth();
        this.screenHeight = Gdx.graphics.getHeight();

        this.camera = new OrthographicCamera();
        this.viewport = new FitViewport(worldHeight, worldWidth, camera);

        this.shapeRenderer = new ShapeRenderer();
    }

    public void initialize() {
        Gdx.gl.glClearColor(0, 0, 0, 1);
        updateDimensions();
    }

    public void reset() {
        Gdx.gl.glClear(GL20.GL_COLOR_BUFFER_BIT);
    }

    public void setScreenSize(int width, int height) {
        screenWidth = width;
        screenHeight = height;
        updateDimensions();
    }

    public void dispose() {
        shapeRenderer.dispose();
    }

    public ShapeRenderer getShapeRenderer() {
        return shapeRenderer;
    }

    private void updateDimensions() {
        viewport.setWorldSize(worldWidth, worldHeight);
        viewport.update(screenWidth, screenHeight, true);

        shapeRenderer.setProjectionMatrix(camera.combined);
    }

}
