package com.me.asteroids.systems;

import com.badlogic.gdx.math.Rectangle;
import com.badlogic.gdx.math.Vector2;
import com.me.asteroids.Constants;
import com.me.asteroids.components.ModelComponent;
import com.me.asteroids.components.PositionComponent;
import com.me.asteroids.events.ScreenWrapEvent;
import com.me.common.ecs.ComponentMapper;
import com.me.common.ecs.Engine;
import com.me.common.ecs.Entity;
import com.me.common.ecs.EntitySystem;

public class ScreenWrapSystem extends EntitySystem {

    private ComponentMapper<PositionComponent> positionMapper;
    private ComponentMapper<ModelComponent> modelMapper;

    public ScreenWrapSystem(Engine engine) {
        super(engine, PositionComponent.class, ModelComponent.class);
        positionMapper = engine.getComponentMapper(PositionComponent.class);
        modelMapper = engine.getComponentMapper(ModelComponent.class);
    }

    private void updatePosition(Entity entity, Vector2 position, float newX, float newY) {
        ScreenWrapEvent event = new ScreenWrapEvent(entity);
        engine.callEvent(event);
        if (!event.isCancelled()) {
            position.set(newX, newY);
        }
    }


    @Override
    public void processEntity(Entity entity, float dt) {
        Vector2 position = positionMapper.get(entity).position;
        Rectangle aabb = modelMapper.get(entity).model.getBoundingBox();

        // Check top/bottom edges
        float minY = aabb.y;
        float maxY = minY + aabb.height;
        if (minY > Constants.HEIGHT) {
            updatePosition(entity, position, position.x, (position.y - maxY) + (minY - Constants.HEIGHT));
            return;
        } else if (maxY < 0) {
            updatePosition(entity, position, position.x, Constants.HEIGHT + (position.y - minY) + maxY);
            return;
        }

        // Check right/left edges
        float minX = aabb.x;
        float maxX = minX + aabb.width;
        if (maxX < 0) {
            updatePosition(entity, position, Constants.WIDTH + (position.x - minX) + maxX, position.y);
        }  else if (minX > Constants.WIDTH) {
            updatePosition(entity, position, (position.x - maxX) + (minX - Constants.WIDTH), position.y);
        }
    }

}
